@@ -117,11 +117,17 @@ class BioDataReader(object):
                              positive_label)
 
     def _extract_data(self, data_file, delimiter, samples_on, variable_remover):
-        raw_data = np.loadtxt(data_file, dtype='S', delimiter=delimiter)
+        try:
+            raw_data = np.genfromtxt(data_file, dtype='S', delimiter=delimiter,
+                                     autostrip=True, invalid_raise=True)
+        except ValueError, e:
+            message = 'Some errors were detected reading data file\n'
+            message += '\n'.join(e.message.split('\n')[1:])
+            raise L1L2SignatureException(message)
 
         self.data = raw_data[1:,1:].astype(np.float)
-        self.samples = np.array([x.strip() for x in raw_data[0,1:]])
-        self.variables = np.array([x.strip() for x in raw_data[1:,0]])
+        self.samples = raw_data[0,1:]
+        self.variables = raw_data[1:,0]
 
         if samples_on == 'row':
             self.samples, self.variables =  self.variables, self.samples
@@ -137,9 +143,16 @@ class BioDataReader(object):
 
     def _extract_labels(self, labels_file, delimiter, sample_remover,
                         positive_label):
-        raw_labels = np.loadtxt(labels_file, dtype='S', delimiter=delimiter)
+        try:
+            raw_labels = np.genfromtxt(labels_file, dtype='S',
+                                       delimiter=delimiter,
+                                       autostrip=True, invalid_raise=True)
+        except ValueError, e:
+            message = 'Some errors were detected reading labels file\n'
+            message += '\n'.join(e.message.split('\n')[1:])
+            raise L1L2SignatureException(message)
 
-        labels_samples = dict((ls.strip(), i)
+        labels_samples = dict((ls, i)
                                 for i, ls in enumerate(raw_labels[1:,0]))
         try:
             order = np.array([labels_samples[ls] for ls in self.samples])
@@ -147,7 +160,7 @@ class BioDataReader(object):
             raise L1L2SignatureException('labels mismatch.')
 
         # Labels management ---
-        labels = np.asarray([raw_labels[1:,1][o].strip() for o in order])
+        labels = np.asarray([raw_labels[1:,1][o] for o in order])
 
         # Filtering is performed before the mapping!
         if not sample_remover is None:
@@ -230,8 +243,20 @@ class RangesScaler(object):
         -------
         tau_range : :class:`numpy.ndarray`
             Scaled tau range.
+
+        Raises
+        ------
+        L1L2SignatureException
+            If trange values are not in the [0, 1) interval
+            (right extreme excluded).
         """
-        return np.asanyarray(trange) * self.tau_scaling_factor
+        trange = np.sort(trange)
+
+        if max(trange) >= 1.0 or min(trange) < 0.0:
+            raise L1L2SignatureException('relative tau values have to '
+                                         'be in [0, 1)')
+
+        return trange * self.tau_scaling_factor
 
     def mu_range(self, mrange):
         """Returns a scaled mu range.
@@ -249,8 +274,19 @@ class RangesScaler(object):
         -------
         mu_range : :class:`numpy.ndarray`
             Scaled mu range.
+
+        Raises
+        ------
+        L1L2SignatureException
+            If mrange values are not all greater than 0.
         """
-        return np.asanyarray(mrange) * self.mu_scaling_factor
+        mrange = np.sort(mrange)
+
+        if min(mrange) < 0.0:
+            raise L1L2SignatureException('relative mu values have to be '
+                                         'greater than 0')
+
+        return np.sort(mrange) * self.mu_scaling_factor
 
     @property
     def tau_scaling_factor(self):
